﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.Controls;
using Infragistics.Controls.Charts;
using Infragistics.Windows;

namespace ChartInfoStrips
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            this.DataContext = new MainWindowViewModel();
            Loaded += MainWindow_Loaded;
        }

        void MainWindow_Loaded(object sender, RoutedEventArgs e)
        {
            string format = string.Empty;
            DataTable dt = ((MainWindowViewModel)this.DataContext).SourceData;
            xAxis.ItemsSource = dt.DefaultView;

            LineSeries series1 = new LineSeries();
            series1.ItemsSource = dt.DefaultView;
            series1.ValueMemberPath = "YAxis";
            series1.XAxis = xAxis;
            series1.YAxis = yAxis;
            series1.MarkerType = MarkerType.None;
            series1.Title = "My Line Series";
            if (((MainWindowViewModel)this.DataContext).NumericFormatIsChecked)
                format = "{0:N2}";
            else
                format = "{0:N2}%";

            series1.ToolTip = CreateToolTip(format);
            xamDataChart1.Series.Add(series1);

            //Black line at the origin of the Y Axis
            ValueOverlay origin = new ValueOverlay();
            origin.Axis = yAxis;
            origin.Value = 0.0;
            origin.Thickness = 0.75;
            origin.Brush = new SolidColorBrush(Colors.Black);
            origin.LegendItemVisibility = Visibility.Collapsed;
            xamDataChart1.Series.Add(origin);

            xamDataChart1.Series.Add(CreateOverlay(13));
            xamDataChart1.Series.Add(CreateOverlay(33));
            xamDataChart1.Series.Add(CreateOverlay(52));
            xamDataChart1.Series.Add(CreateOverlay(74));
            xamDataChart1.Series.Add(CreateOverlay(95));
            xamDataChart1.Series.Add(CreateOverlay(116));
            xamDataChart1.Series.Add(CreateOverlay(137));
            xamDataChart1.Series.Add(CreateOverlay(158));
            xamDataChart1.Series.Add(CreateOverlay(180));
            xamDataChart1.Series.Add(CreateOverlay(201));
            xamDataChart1.Series.Add(CreateOverlay(223));
            xamDataChart1.Series.Add(CreateOverlay(244));
        }

        private ValueOverlay CreateOverlay(int value = 0)
        {
            ValueOverlay overlay = new ValueOverlay();
            overlay.Axis = hiddenYearXAxis;
            overlay.Value = value;
            overlay.Thickness = 0.75;
            overlay.LegendItemVisibility = Visibility.Collapsed;
            overlay.Brush = new SolidColorBrush(Colors.Black);
            return overlay;
        }


        /// <summary>
        /// 
        /// </summary>
        /// <param name="format"></param>
        /// <returns></returns>
        private object CreateToolTip(string format)
        {
            Grid grd = new Grid();
            RowDefinition row = new RowDefinition();
            row.Height = GridLength.Auto;
            grd.RowDefinitions.Add(row);
            row = new RowDefinition();
            row.Height = GridLength.Auto;
            row = new RowDefinition();
            grd.RowDefinitions.Add(row);


            ColumnDefinition col = new ColumnDefinition();
            col.Width = GridLength.Auto;
            grd.ColumnDefinitions.Add(col);
            col = new ColumnDefinition();
            col.Width = GridLength.Auto;
            grd.ColumnDefinitions.Add(col);

            Border bdr = new Border();
            bdr.BorderBrush = Brushes.Gray;
            bdr.SetValue(Grid.RowProperty, 0);
            bdr.SetValue(Grid.ColumnProperty, 0);
            bdr.SetValue(Grid.RowSpanProperty, 1);
            bdr.SetValue(Grid.ColumnSpanProperty, 2);
            bdr.BorderThickness = new Thickness(0, 0, 0, 1);
            bdr.Opacity = .1;
            grd.Children.Add(bdr);

            TextBlock tbValue = new TextBlock();
            tbValue.Text = "XAxis:";
            tbValue.SetValue(Grid.RowProperty, 0);
            tbValue.SetValue(Grid.ColumnProperty, 0);
            grd.Children.Add(tbValue);

            TextBlock txtValue = new TextBlock();
            Binding binding = new Binding("Item.XAxis");
            txtValue.SetBinding(TextBlock.TextProperty, binding);
            txtValue.SetValue(Grid.RowProperty, 0);
            txtValue.SetValue(Grid.ColumnProperty, 0);
            txtValue.SetValue(Grid.ColumnSpanProperty, 2);
            txtValue.Margin = new Thickness(35, 01, 05, 01);
            grd.Children.Add(txtValue);

            tbValue = new TextBlock();
            tbValue.Text = "YAxis: ";
            tbValue.SetValue(Grid.RowProperty, 1);
            tbValue.SetValue(Grid.ColumnProperty, 0);
            grd.Children.Add(tbValue);

            // Setup a datatrigger to change the StringFormat based on the NumericFormatIsChecked property.
            DataTrigger stringFormatTrigger = new DataTrigger
            {
                Binding = new Binding("NumericFormatIsChecked") { Source = this.DataContext },
                Value = false,
            };
            stringFormatTrigger.Setters.Add(new Setter { Property = TextBlock.TextProperty, Value = new Binding("Item.YAxis") { StringFormat = "{0:N2}%" } });

            // since the tooltip is not a DataTemplate, add the data trigger to a style and set the style on the TextBlock
            Style yAxisTextStyle = new System.Windows.Style(typeof(TextBlock));
            yAxisTextStyle.Setters.Add(new Setter(TextBlock.TextProperty, new Binding("Item.YAxis") { StringFormat = format }));
            yAxisTextStyle.Triggers.Add(stringFormatTrigger);

            txtValue = new TextBlock();
            txtValue.Style = yAxisTextStyle;
            txtValue.SetValue(Grid.RowProperty, 1);
            txtValue.SetValue(Grid.ColumnProperty, 0);
            txtValue.SetValue(Grid.ColumnSpanProperty, 2);
            txtValue.Margin = new Thickness(35, 0, 05, 0);
            grd.Children.Add(txtValue);

            return grd;
        }

        double height = 0;
        private void xamDataChart_RefreshCompleted(object sender, EventArgs e)
        {
            if (chtWeek.Series.Any())
            {
                // doesn't really matter which series we use for this
                Series series = chtWeek.Series[0];
                if (series.RootCanvas != null)
                {
                    HorizontalAxisLabelPanel axisPanel = (HorizontalAxisLabelPanel)Utilities.GetDescendantFromType<HorizontalAxisLabelPanel>(chtWeek, false, SearchHorizontalAxisCallback);
                }
            }
        }

        private bool SearchHorizontalAxisCallback(HorizontalAxisLabelPanel dependencyObject)
        {
            // doesn't really matter which series we use for this
            Series series = chtWeek.Series[0];
            if (series.RootCanvas != null)
            {
                double pixel = PriceYAxis.ScaleValue(0.0);

                // move the X axis to the correct location
                double axisHeightAdjustment = series.RootCanvas.ActualHeight - pixel;

                // if the height is moved up sufficiently, switch to inside bottom.
                if (axisHeightAdjustment > dependencyObject.ActualHeight)
                {
                    PriceXAxis.LabelSettings.Location = AxisLabelsLocation.InsideBottom;
                    axisHeightAdjustment = axisHeightAdjustment - dependencyObject.ActualHeight;
                }

                dependencyObject.RenderTransform = new TranslateTransform(0.0, -axisHeightAdjustment);
            }

            return false;
        }

        private void HorizontalScroll_Scroll(object sender, System.Windows.Controls.Primitives.ScrollEventArgs e)
        {
            xamDataChart1.HorizontalZoombar.Range.FromScaleScroll(xamDataChart1.WindowScaleHorizontal, HorizontalScroll.Value);
        }
    }

    public class MainWindowViewModel : INotifyPropertyChanged
    {
        /// <summary>
        /// 
        /// </summary>
        private double weekMaximumValue;

        /// <summary>
        ///  gets or sets 
        /// </summary>
        public double WeekMaximumValue
        {
            get
            {
                return this.weekMaximumValue;
            }
            set
            {
                if (this.weekMaximumValue != value)
                {
                    this.weekMaximumValue = value;
                    this.OnPropertyChanged("WeekMaximumValue");
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private double weekMinimumValue;

        /// <summary>
        ///  gets or sets 
        /// </summary>
        public double WeekMinimumValue
        {
            get
            {
                return this.weekMinimumValue;
            }
            set
            {
                if (this.weekMinimumValue != value)
                {
                    this.weekMinimumValue = value;
                    this.OnPropertyChanged("WeekMinimumValue");
                }
            }
        }

        /// <summary>
        /// 
        /// </summary>
        private double weekInterval;

        /// <summary>
        ///  gets or sets 
        /// </summary>
        public double WeekInterval
        {
            get
            {
                return this.weekInterval;
            }
            set
            {
                if (this.weekInterval != value)
                {
                    this.weekInterval = value;
                    this.OnPropertyChanged("WeekInterval");
                }
            }
        }
        private bool _numericFormatIsChecked = true;
        public bool NumericFormatIsChecked
        {
            get { return _numericFormatIsChecked; }
            set
            {
                if (_numericFormatIsChecked != value)
                {
                    _numericFormatIsChecked = value;
                    OnPropertyChanged("NumericFormatIsChecked");
                }
            }
        }

        public DataTable SourceData
        {
            get { return LoadData(); }
        }

        List<string> workingDays = new List<string>() { "Mon", "Tue", "Wed", "Thr", "Fri" };

        public DataTable DaysOfWeek
        {
            get
            {
                DataTable dt = new DataTable();
                dt.Columns.Add("Name", typeof(string));
                dt.Columns.Add("Value", typeof(double));

                Random random = new Random();

                foreach (string item in workingDays)
                {
                    dt.Rows.Add(item, random.Next(2,500));
                }

                if (dt.Rows.Count > 0)
                {
                    var data = dt.AsEnumerable();
                    double max = Math.Ceiling(data.Where(v => !object.Equals(v["Value"], double.NaN)).Max(v => (double)v["Value"]));
                    double min = Math.Ceiling(data.AsEnumerable().Where(v => !object.Equals(v["Value"], double.NaN)).Min(v => (double)v["Value"]));

                    this.WeekMaximumValue = max;
                    this.WeekMinimumValue = min - 1;
                    this.WeekInterval = Math.Ceiling((max - min) / 10);
                }
                return dt;
            }
        }

        private DataTable LoadData()
        {
            DataTable dt = new DataTable();
            dt.Columns.Add("XAxis", typeof(string));
            dt.Columns.Add("YAxis", typeof(double));

            Random random = new Random();
            double max = 0.2;
            double min = -0.1;
            var directory = System.AppDomain.CurrentDomain.BaseDirectory;
            string path = directory.Replace(@"\bin\Debug\", string.Empty);
            DataSet ds = new DataSet();
            ds.ReadXml(string.Concat(path, @"\MetaData\DaysOfYearMetaData.xml"));
            DataTable newTable = ds.Tables[0];

            foreach (DataRow dr in newTable.Rows)
            {
                dt.Rows.Add(dr["Label"], random.NextDouble() * (max - min) + min);
            }

            return dt;
        }


        #region INotifyPropertyChanged
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = this.PropertyChanged;
            if (handler != null)
            {
                var e = new PropertyChangedEventArgs(propertyName);
                handler(this, e);
            }
        }
        #endregion
    }
}
